
# q-expansion of newform 936.2.ba.b, downloaded from the LMFDB on 27 December 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 936
weight = 2
poly_data = [1, 0, 0, 0, 107, 0, 0, 0, 27, 0, 0, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[145, 0, 0, 0, 38, 0, 0, 0, 1, 0, 0, 0], 76], [[0, 373, 0, 0, 0, 38, 0, 0, 0, 1, 0, 0], 76], [[0, -711, 0, 0, 0, -190, 0, 0, 0, -7, 0, 0], 76], [[-24, 0, 245, 0, -38, 0, 76, 0, -2, 0, 3, 0], 76], [[-24, 0, -245, 0, -38, 0, -76, 0, -2, 0, -3, 0], 76], [[0, 0, 787, 0, 0, 0, 190, 0, 0, 0, 7, 0], 76], [[0, 0, -749, 0, 0, 0, -190, 0, 0, 0, -7, 0], 38], [[0, -245, 0, 2530, 0, -76, 0, 646, 0, -3, 0, 24], 76], [[0, -245, 0, -2530, 0, -76, 0, -646, 0, -3, 0, -24], 76], [[0, 0, 0, -3859, 0, 0, 0, -950, 0, 0, 0, -35], 76], [[0, 0, 0, -7345, 0, 0, 0, -1862, 0, 0, 0, -69], 76]]

hecke_ring_character_values = [[703, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [469, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [209, [-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [145, [0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 1, 0, 0, 0, -1, 0, -1, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1], [0, 0, 0, 0, 0, 1, 2, -1, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0], [1, -2, 0, 0, 1, 0, -1, -2, 0, 0, 0, 0], [0, 0, -1, -2, 0, 0, 0, 0, 2, 0, -1, 2], [0, 0, 0, -1, 0, 0, 0, 0, 0, 1, 0, -1], [-1, 2, 0, 0, -3, 0, 1, 2, 0, 0, 0, 0], [3, 2, 0, 0, 0, 0, 3, -2, 0, 0, 0, 0], [0, 0, 1, -2, 0, 0, 0, 0, 1, 1, 0, 0], [0, 0, 0, 0, -1, 1, 6, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 3], [0, 0, 1, -1, 0, 0, 0, 0, 0, -1, -1, -1], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -3, 2], [4, 4, 0, 0, -3, -3, 0, 0, 0, 0, 0, 0], [0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 0, 0], [0, 0, -5, 0, 0, 0, 0, 0, 2, 2, 0, 0], [0, 3, 0, 0, 0, 0, 0, -3, 0, 0, 0, 0], [-6, 2, 0, 0, -2, -2, 0, 0, 0, 0, 0, 0], [0, 0, -1, 2, 0, 0, 0, 0, 2, 2, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, -2, 2, 3, -4], [6, -1, 0, 0, 4, 0, -6, -1, 0, 0, 0, 0], [0, 0, -2, 3, 0, 0, 0, 0, -3, 0, -2, -3], [0, 0, 0, 0, 1, -1, 0, 2, 0, 0, 0, 0], [0, 0, 4, 1, 0, 0, 0, 0, 0, -6, -4, 1], [6, -3, 0, 0, 4, 0, -6, -3, 0, 0, 0, 0], [0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1], [0, 0, 0, 0, 1, -1, -6, 4, 0, 0, 0, 0], [0, 0, -1, 2, 0, 0, 0, 0, 0, -2, 1, 2], [0, 0, 0, 0, 0, 0, 0, 0, -1, 1, -5, -6], [-2, 2, 0, 0, -4, -4, 0, 0, 0, 0, 0, 0], [0, 0, -1, -4, 0, 0, 0, 0, 5, 5, 0, 0], [-13, 0, 0, 0, 0, -1, -13, 0, 0, 0, 0, 0], [-6, 2, 0, 0, 2, 2, 0, 0, 0, 0, 0, 0], [-3, 0, 0, 0, 0, -5, -3, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, -4, 4, 0, 3], [0, 0, 3, 2, 0, 0, 0, 0, -9, 0, 3, -2], [0, 0, 2, 4, 0, 0, 0, 0, 2, 0, 2, -4], [0, 0, 0, 0, -6, 6, 2, 10, 0, 0, 0, 0], [0, 0, 3, -3, 0, 0, 0, 0, 0, 6, -3, -3], [-1, 0, 0, 0, 0, -8, -1, 0, 0, 0, 0, 0], [0, 0, 3, 6, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 5, -5, 8, -2, 0, 0, 0, 0], [-4, 4, 0, 0, -2, -2, 0, 0, 0, 0, 0, 0], [7, 2, 0, 0, 1, 0, -7, 2, 0, 0, 0, 0], [0, 0, 2, 7, 0, 0, 0, 0, -2, -2, 0, 0], [-6, -7, 0, 0, 0, 2, -6, 7, 0, 0, 0, 0], [0, 0, -2, 5, 0, 0, 0, 0, -9, 0, -2, -5], [0, 0, -5, -6, 0, 0, 0, 0, 6, 6, 0, 0], [-6, 1, 0, 0, 0, -4, -6, -1, 0, 0, 0, 0], [0, 0, 0, 7, 0, 0, 0, 0, -8, 0, 0, -7], [0, 0, -2, -5, 0, 0, 0, 0, 9, 0, -2, 5], [0, 0, -1, -6, 0, 0, 0, 0, 0, -4, 1, -6], [0, 0, 1, -2, 0, 0, 0, 0, 0, 1, -1, -2], [4, -5, 0, 0, 0, 5, 4, 5, 0, 0, 0, 0], [0, 0, 0, 0, 4, -4, 12, -2, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 9, -9, -3, -2], [0, 0, 0, 0, 1, -1, -4, -6, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, -3, 3, -1, 8], [-12, -1, 0, 0, 0, 3, -12, 1, 0, 0, 0, 0], [0, 0, -1, -4, 0, 0, 0, 0, -2, 0, -1, 4], [-8, -10, 0, 0, 3, 3, 0, 0, 0, 0, 0, 0], [0, 0, -1, 4, 0, 0, 0, 0, -2, -2, 0, 0], [12, 5, 0, 0, -11, 0, -12, 5, 0, 0, 0, 0], [0, 0, 0, 0, 7, -7, -12, -10, 0, 0, 0, 0], [0, 0, -3, 1, 0, 0, 0, 0, 0, 16, 3, 1], [5, 4, 0, 0, 0, -6, 5, -4, 0, 0, 0, 0], [0, 0, -3, 0, 0, 0, 0, 0, 11, 11, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, -6, 6, -2, -9], [14, 6, 0, 0, 2, 2, 0, 0, 0, 0, 0, 0], [12, -6, 0, 0, -1, -1, 0, 0, 0, 0, 0, 0], [-5, 6, 0, 0, -13, 0, 5, 6, 0, 0, 0, 0], [0, 0, 2, -11, 0, 0, 0, 0, 8, 8, 0, 0], [0, 0, 0, -8, 0, 0, 0, 0, -3, 0, 0, 8], [5, 0, 0, 0, 0, 8, 5, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, -4, 4, -1, 10], [4, 5, 0, 0, -8, 0, -4, 5, 0, 0, 0, 0], [0, 0, -1, 2, 0, 0, 0, 0, 0, -14, 1, 2], [6, -9, 0, 0, 8, 0, -6, -9, 0, 0, 0, 0], [0, 0, -2, -7, 0, 0, 0, 0, -4, -4, 0, 0], [0, 0, 0, 0, 1, -1, 22, -4, 0, 0, 0, 0], [0, 0, 0, 0, -5, 5, 10, -4, 0, 0, 0, 0], [0, 0, -4, -1, 0, 0, 0, 0, 0, -6, 4, -1], [0, 0, 0, 0, 0, 0, 0, 0, -4, 4, 5, 6], [-2, 7, 0, 0, -10, 0, 2, 7, 0, 0, 0, 0], [0, 0, 3, 0, 0, 0, 0, 0, 10, 10, 0, 0], [-20, -1, 0, 0, 0, 5, -20, 1, 0, 0, 0, 0], [0, 0, 0, 3, 0, 0, 0, 0, 12, 0, 0, -3], [0, 0, 0, 0, 0, 0, 0, 0, -10, 10, -1, -6], [7, 4, 0, 0, -9, 0, -7, 4, 0, 0, 0, 0], [0, 0, 2, -3, 0, 0, 0, 0, 8, 0, 2, 3], [0, -13, 0, 0, 3, 0, 0, -13, 0, 0, 0, 0], [0, 0, -4, 1, 0, 0, 0, 0, 0, 8, 4, 1], [0, 0, -11, -8, 0, 0, 0, 0, 6, 6, 0, 0], [0, 0, 0, -3, 0, 0, 0, 0, 0, -7, 0, -3], [4, -16, 0, 0, 6, 6, 0, 0, 0, 0, 0, 0], [0, 9, 0, 0, 0, -4, 0, -9, 0, 0, 0, 0], [-2, 14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, -5, 5, -7, -8], [0, 0, -1, -7, 0, 0, 0, 0, -2, 0, -1, 7], [0, 0, 4, 1, 0, 0, 0, 0, -17, 0, 4, -1], [0, 0, 0, 0, 5, -5, 22, -8, 0, 0, 0, 0], [21, 4, 0, 0, -2, 0, -21, 4, 0, 0, 0, 0], [0, 0, -1, 10, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 18, -18, -1, 6], [0, 0, -4, -3, 0, 0, 0, 0, 0, 0, 4, -3], [-16, 2, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0], [18, 6, 0, 0, 6, 6, 0, 0, 0, 0, 0, 0], [-7, -4, 0, 0, -4, 0, 7, -4, 0, 0, 0, 0], [0, 0, 3, 2, 0, 0, 0, 0, 9, 9, 0, 0], [-12, -1, 0, 0, 0, -1, -12, 1, 0, 0, 0, 0], [-1, 4, 0, 0, 0, -13, -1, -4, 0, 0, 0, 0], [0, 0, 5, -1, 0, 0, 0, 0, -11, 0, 5, 1], [14, -3, 0, 0, -1, 0, -14, -3, 0, 0, 0, 0], [0, 0, -5, -1, 0, 0, 0, 0, 20, 0, -5, 1], [0, 0, 6, -11, 0, 0, 0, 0, 0, 7, -6, -11], [0, 0, -3, 1, 0, 0, 0, 0, 0, 12, 3, 1], [3, -2, 0, 0, 0, -6, 3, 2, 0, 0, 0, 0], [0, 0, 0, 0, -2, 2, -4, 6, 0, 0, 0, 0], [0, 0, -6, 6, 0, 0, 0, 0, 0, -11, 6, 6], [0, 0, 0, 0, 0, 0, 0, 0, -6, 6, 0, -3], [8, -3, 0, 0, 1, 0, -8, -3, 0, 0, 0, 0], [0, 0, 2, 4, 0, 0, 0, 0, -15, 0, 2, -4], [-14, -1, 0, 0, 0, -14, -14, 1, 0, 0, 0, 0], [0, 0, 10, 5, 0, 0, 0, 0, -10, 0, 10, -5], [0, 0, 0, 0, -15, 15, 4, 22, 0, 0, 0, 0], [-2, 15, 0, 0, -20, 0, 2, 15, 0, 0, 0, 0], [0, -5, 0, 0, 0, -9, 0, 5, 0, 0, 0, 0], [0, 0, 1, -2, 0, 0, 0, 0, -10, -10, 0, 0], [0, 0, 0, 0, -5, 5, 8, 18, 0, 0, 0, 0], [-2, -4, 0, 0, 11, 11, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, -5, 5, -1, -8], [-1, -6, 0, 0, 22, 0, 1, -6, 0, 0, 0, 0], [0, 0, 3, 12, 0, 0, 0, 0, 3, 3, 0, 0], [2, -13, 0, 0, 0, 21, 2, 13, 0, 0, 0, 0], [0, 0, 4, 6, 0, 0, 0, 0, -13, 0, 4, -6], [0, 0, -7, -12, 0, 0, 0, 0, 0, 9, 7, -12], [1, -4, 0, 0, 23, 0, -1, -4, 0, 0, 0, 0], [0, 0, 3, -2, 0, 0, 0, 0, 13, 13, 0, 0], [0, 0, 0, 0, -1, 1, 22, 12, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 4, -4, -6, 7], [0, 0, 0, 0, -1, 1, -14, -2, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 20, -20, -9, 0], [-5, 0, 0, 0, 0, -6, -5, 0, 0, 0, 0, 0], [0, 0, 5, 12, 0, 0, 0, 0, -9, 0, 5, -12], [-18, -18, 0, 0, -4, -4, 0, 0, 0, 0, 0, 0], [0, 0, -16, 5, 0, 0, 0, 0, 8, 8, 0, 0], [13, -2, 0, 0, 4, 0, -13, -2, 0, 0, 0, 0], [0, 0, -4, 7, 0, 0, 0, 0, 1, 0, -4, -7], [0, 0, 0, 0, -3, 3, 12, 18, 0, 0, 0, 0], [0, 0, 4, 2, 0, 0, 0, 0, 0, -22, -4, 2], [0, 0, 0, 0, 7, -7, -22, -4, 0, 0, 0, 0], [0, 0, 0, 6, 0, 0, 0, 0, 0, -14, 0, 6], [-6, -22, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, -9, -18, 0, 0, 0, 0, 11, 11, 0, 0], [24, 16, 0, 0, 5, 5, 0, 0, 0, 0, 0, 0], [0, 0, -1, 6, 0, 0, 0, 0, -19, -19, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 2, -2, 3, 10], [0, 0, -5, 14, 0, 0, 0, 0, -13, 0, -5, -14], [9, 10, 0, 0, 3, 0, -9, 10, 0, 0, 0, 0], [0, 0, -9, -1, 0, 0, 0, 0, 0, 32, 9, -1], [0, 0, 1, 14, 0, 0, 0, 0, -4, -4, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, -24, 24, 1, -4], [-8, -16, 0, 0, -2, -2, 0, 0, 0, 0, 0, 0], [2, 24, 0, 0, -6, -6, 0, 0, 0, 0, 0, 0]]
